;;;NoBeam
(d-DirectivePut-chord-postfix "NoBeam" "\\noBeam")
(d-DirectivePut-chord-display "NoBeam" "noBeam")
(d-DirectivePut-chord-graphic_name "NoBeam" "NoBeam")
(d-DirectivePut-chord-override "NoBeam" 32)
(d-RefreshDisplay)
;;;End of scheme script