  ;;;;;;;;;;;;;;
(d-DirectivePut-movementcontrol-override "MvntComment" (logior DENEMO_OVERRIDE_GRAPHIC DENEMO_OVERRIDE_EDITOR))
(d-DirectivePut-movementcontrol-display "MvntComment" "Click to edit comment")

