;;;Upprall
;(d-DirectivePut-chord-graphic "UpPrall" "\xEE\x86\x90 Emmentaler 28")
(ToggleChordDirective "UpPrall" LG-UpPrall "\\upprall" DENEMO_OVERRIDE_ABOVE)
(d-DirectivePut-chord-gy "UpPrall" -100)
