/******** generated automatically from generate_source. See generate_source.c */
{"CursorLeft", NULL, "Cursor Left", "","Moves the cursor one object left, altering the selection if any",G_CALLBACK (cursorleft_cb)},
{"MoveCursorLeft", NULL, "Move Cursor Left", "","Moves the cursor one object left, without altering the selection",G_CALLBACK (movecursorleft_cb)},
{"CursorDown", NULL, "Cursor Down", "","Moves the cursor one scale step down",G_CALLBACK (cursordown_cb)},
{"CursorUp", NULL, "Cursor Up", "","Moves the cursor one scale step up",G_CALLBACK (cursorup_cb)},
{"CursorRight", NULL, "Cursor Right", "","Moves the cursor one object right, altering the selection if any",G_CALLBACK (cursorright_cb)},
{"MoveCursorRight", NULL, "Move Cursor Right", "","Moves the cursor one object right, without altering the selection",G_CALLBACK (movecursorright_cb)},
{"GoToMark", NULL, "To Mark", "","Moves the cursor to the Mark without altering the selection",G_CALLBACK (goto_mark)},
{"SwapPointAndMark", NULL, "Swap Ends of Selection", "","Swaps the active end of the selection",G_CALLBACK (swap_point_and_mark)},
{"GoToSelectionStart", NULL, "To Selection Start", "","Moves the cursor to the first object in the selection without altering the selection. returns #f if no selection",G_CALLBACK (goto_selection_start)},
{"PushPosition", NULL, "Push Position", "","Pushes the current cursor position onto a stack",G_CALLBACK (PushPosition)},
{"PopPosition", NULL, "Pop Position", "","Pops a position from the stack of cursor positions, moving the cursor there",G_CALLBACK (PopPosition)},
{"PopPushPosition", NULL, "Pop and Push Position", "","Pops a position from the stack of cursor positions, pushes the current position, then moves the cursor to the popped position",G_CALLBACK (PopPushPosition)},
{"ToggleReduceToDrawingArea", NULL, "Hide/Show Menus", "","Hides/Shows menus, panes etc. The ones shown are those checked in the view menu.",G_CALLBACK (ToggleReduceToDrawingArea)},
{"StaffUp", NULL, "Staff Up", "","Moves the cursor to the staff above, extending selection if any",G_CALLBACK (staffup_cb)},
{"StaffDown", NULL, "Staff Down", "","Moves the cursor to the staff below, extending selection if any",G_CALLBACK (staffdown_cb)},
{"MoveToStaffUp", NULL, "Move to Staff Up", "","Moves the cursor to the staff above without altering selection",G_CALLBACK (movetostaffup_cb)},
{"MoveToStaffDown", NULL, "Move to Staff Down", "","Moves the cursor to the staff below  without altering selection",G_CALLBACK (movetostaffdown_cb)},
{"MeasureLeft", NULL, "Measure Left", "","Moves the cursor to the first object in the next measure, extending selection if any",G_CALLBACK (measureleft_cb)},
{"MeasureRight", NULL, "Measure Right", "","Moves the cursor to the first object in the previous measure, extending selection if any",G_CALLBACK (measureright_cb)},
{"MoveToMeasureLeft", NULL, "Move to Measure Left", "","Moves the cursor to the first object in the next measure leaving selection, if any, unchanged",G_CALLBACK (movetomeasureleft_cb)},
{"MoveToMeasureRight", NULL, "Move to Measure Right", "","Moves the cursor to the first object in the previous measureleaving selection, if any, unchanged",G_CALLBACK (movetomeasureright_cb)},
{"A", NULL, "A", "","Append/Edit  A",G_CALLBACK (go_to_A_key_cb)},
{"B", NULL, "B", "","Append/Edit  B",G_CALLBACK (go_to_B_key_cb)},
{"C", NULL, "C", "","Append/Edit  C",G_CALLBACK (go_to_C_key_cb)},
{"D", NULL, "D", "","Append/Edit  D",G_CALLBACK (go_to_D_key_cb)},
{"E", NULL, "E", "","Append/Edit  E",G_CALLBACK (go_to_E_key_cb)},
{"F", NULL, "F", "","Append/Edit  F",G_CALLBACK (go_to_F_key_cb)},
{"G", NULL, "G", "","Append/Edit  G",G_CALLBACK (go_to_G_key_cb)},
{"OctaveUp", NULL, "Octave Up", "","Octave Up",G_CALLBACK (octave_up_key_cb)},
{"OctaveDown", NULL, "Octave Down", "","Octave Down",G_CALLBACK (octave_down_key_cb)},
{"WholeNote", NULL, "WholeNote", "","Insert \xF0\x9D\x85\x9D",G_CALLBACK (insert_chord_0key_cb)},
{"HalfNote", NULL, "HalfNote", "","Insert \xF0\x9D\x85\x9E",G_CALLBACK (insert_chord_1key_cb)},
{"QuarterNote", NULL, "QuarterNote", "","Insert \xF0\x9D\x85\x9F",G_CALLBACK (insert_chord_2key_cb)},
{"EighthNote", NULL, "EighthNote", "","Insert \xF0\x9D\x85\xA0",G_CALLBACK (insert_chord_3key_cb)},
{"SixteenthNote", NULL, "SixteenthNote", "","Insert \xF0\x9D\x85\xA1",G_CALLBACK (insert_chord_4key_cb)},
{"ThirtysecondNote", NULL, "ThirtysecondNote", "","Insert \xF0\x9D\x85\xA2",G_CALLBACK (insert_chord_5key_cb)},
{"SixtyfourthNote", NULL, "SixtyfourthNote", "","Insert \xF0\x9D\x85\xA3",G_CALLBACK (insert_chord_6key_cb)},
{"OneHundredTwentyEighthNote", NULL, "OneHundredTwentyEighthNote", "","Insert \xF0\x9D\x85\xA4",G_CALLBACK (insert_chord_7key_cb)},
{"TwoHundredFiftySixthNote", NULL, "TwoHundredFiftySixthNote", "","Insert \xF0\x9D\x85\xA5",G_CALLBACK (insert_chord_8key_cb)},
{"InsertWholeRest", NULL, "<span font_desc=\"Denemo\">\xF0\x9D\x84\xBB</span>", "","Insert \xF0\x9D\x84\xBB rest",G_CALLBACK (insert_rest_0key_cb)},
{"InsertHalfRest", NULL, "<span font_desc=\"Denemo\">\xF0\x9D\x84\xBC</span>", "","Insert \xF0\x9D\x84\xBC rest",G_CALLBACK (insert_rest_1key_cb)},
{"InsertQuarterRest", NULL, "<span font_desc=\"Denemo\">\xF0\x9D\x84\xBD</span>", "","Insert \xF0\x9D\x84\xBD rest",G_CALLBACK (insert_rest_2key_cb)},
{"InsertEighthRest", NULL, "<span font_desc=\"Denemo\">\xF0\x9D\x84\xBE</span>", "","Insert \xF0\x9D\x84\xBE rest",G_CALLBACK (insert_rest_3key_cb)},
{"InsertSixteenthRest", NULL, "<span font_desc=\"Denemo\">\xF0\x9D\x84\xBF</span>", "","Insert \xF0\x9D\x84\xBF rest",G_CALLBACK (insert_rest_4key_cb)},
{"InsertThirtysecondRest", NULL, "<span font_desc=\"Denemo\">\xF0\x9D\x85\x80</span>", "","Insert \xF0\x9D\x85\x80 rest",G_CALLBACK (insert_rest_5key_cb)},
{"InsertSixtyfourthRest", NULL, "<span font_desc=\"Denemo\">\xF0\x9D\x85\x81</span>", "","Insert \xF0\x9D\x85\x81 rest",G_CALLBACK (insert_rest_6key_cb)},
{"InsertBlankWholeNote", NULL, "InsertBlankWholeNote", "","Insert a non-printing \xF0\x9D\x84\xBB rest",G_CALLBACK (insert_blankchord_0key_cb)},
{"InsertBlankHalfNote", NULL, "InsertBlankHalfNote", "","Insert a non-printing \xF0\x9D\x84\xBC rest",G_CALLBACK (insert_blankchord_1key_cb)},
{"InsertBlankQuarterNote", NULL, "InsertBlankQuarterNote", "","Insert a non-printing \xF0\x9D\x84\xBD rest",G_CALLBACK (insert_blankchord_2key_cb)},
{"InsertBlankEighthNote", NULL, "InsertBlankEighthNote", "","Insert a non-printing \xF0\x9D\x84\xBE rest",G_CALLBACK (insert_blankchord_3key_cb)},
{"InsertBlankSixteenthNote", NULL, "InsertBlankSixteenthNote", "","Insert a non-printing \xF0\x9D\x84\xBF rest",G_CALLBACK (insert_blankchord_4key_cb)},
{"InsertBlankThirtysecondNote", NULL, "InsertBlankThirtysecondNote", "","Insert a non-printing \xF0\x9D\x85\x80 rest",G_CALLBACK (insert_blankchord_5key_cb)},
{"InsertBlankSixtyfourthNote", NULL, "InsertBlankSixtyfourthNote", "","Insert a non-printing \xF0\x9D\x85\x81 rest",G_CALLBACK (insert_blankchord_6key_cb)},
{"InsertBlankOneHundredTwentyEighthNote", NULL, "InsertBlankOneHundredTwentyEighthNote", "","Insert a non-printing \xF0\x9D\x85\x82 rest",G_CALLBACK (insert_blankchord_7key_cb)},
{"InsertBlankTwoHundredFiftySixthNote", NULL, "InsertBlankTwoHundredFiftySixthNote", "","Insert a non-printing \xF0\x9D\x85\x83 rest",G_CALLBACK (insert_blankchord_8key_cb)},
{"ToggleRestMode", NULL, "Toggle Rest Mode", "","No Tooltip yet",G_CALLBACK (rest_toggle_key_cb)},
{"ToggleBlankMode", NULL, "Toggle Blank Mode", "","No Tooltip yet",G_CALLBACK (toggle_blank_cb)},
{"InsertDuplet", NULL, "Insert Duplet", "","No Tooltip yet",G_CALLBACK (insert_duplet_cb)},
{"InsertTriplet", NULL, "Insert Triplet", "","No Tooltip yet",G_CALLBACK (insert_triplet_cb)},
{"StartTriplet", NULL, "Start Triplet", "","No Tooltip yet",G_CALLBACK (start_triplet_cb)},
{"EndTuplet", NULL, "End Tuplet", "","No Tooltip yet",G_CALLBACK (end_tuplet_cb)},
{"InsertQuadtuplet", NULL, "Insert Quadtuplet", "","No Tooltip yet",G_CALLBACK (insert_quadtuplet_cb)},
{"InsertQuintuplet", NULL, "Insert Quintuplet", "","No Tooltip yet",G_CALLBACK (insert_quintuplet_cb)},
{"InsertSextuplet", NULL, "Insert Sextuplet", "","No Tooltip yet",G_CALLBACK (insert_sextuplet_cb)},
{"InsertSeptuplet", NULL, "Insert Septuplet", "","No Tooltip yet",G_CALLBACK (insert_septuplet_cb)},
{"AddNoteToChord", NULL, "Add note", "","Add a note to the current chord\nThe cursor position determines which note to add",G_CALLBACK (add_tone_key_cb)},
{"RemoveNoteFromChord", NULL, "Remove note", "","Remove a note from the current chord",G_CALLBACK (remove_tone_key_cb)},
{"Sharpen", NULL, "Sharpen", "","No Tooltip yet",G_CALLBACK (sharpen_key_cb)},
{"Flatten", NULL, "Flatten", "","No Tooltip yet",G_CALLBACK (flatten_key_cb)},
{"PendingSharpen", NULL, "Sharpen Next Note", "","Increases the sharpness of the next entered note. The status bar shows the current state.",G_CALLBACK (pending_sharpen_cb)},
{"PendingFlatten", NULL, "Flatten Next Note", "","Increases the flatness of the next entered note. The status bar shows the current state.",G_CALLBACK (pending_flatten_cb)},
{"StemUp", NULL, "StemUp", "","Alters a StemNeutral object to stem up.",G_CALLBACK (stem_up_cb)},
{"StemDown", NULL, "StemDown", "","Alters a StemNeutral object to stem down.",G_CALLBACK (stem_down_cb)},
{"AddDot", NULL, "Add Dot", "","No Tooltip yet",G_CALLBACK (add_dot_key_cb)},
{"RemoveDot", NULL, "Remove Dot", "","No Tooltip yet",G_CALLBACK (remove_dot_key_cb)},
{"InsertTiedNote", NULL, "Tied note", "","Inserts a duplicate of the current note, tied",G_CALLBACK (tie_notes_key_cb)},
{"ToggleTie", NULL, "Toggle Tie", "","Ties/unties the note at the cursor",G_CALLBACK (toggle_tie)},
{"DeleteObject", NULL, "Delete Object", "","Delete the object at the cursor",G_CALLBACK (deleteobject_cb)},
{"DeletePreviousObject", NULL, "Delete Previous Object", "","Delete to the left of the cursor.",G_CALLBACK (deletepreviousobject_cb)},
{"InsertMeasure", NULL, "Insert Measure Before", "","Insert a blank measure before the current one (in all staffs)",G_CALLBACK (insert_measure_key_cb)},
{"AddMeasure", NULL, "Insert Measure After", "","Insert a blank measure after the current one (in all staffs)",G_CALLBACK (addmeasureafter_cb)},
{"InsertMeasureBefore", NULL, "Staff Insert Measure Before", "","Insert a blank measure before the current one (in current staff)",G_CALLBACK (insertmeasurebefore_cb)},
{"InsertMeasureAfter", NULL, "Staff Insert Measure After", "","Insert a blank measure in current staff after the current measure",G_CALLBACK (insertmeasureafter_cb)},
{"AppendMeasure", NULL, "Staff Append Measure", "","No Tooltip yet",G_CALLBACK (append_measure_key_cb)},
{"DeleteMeasure", NULL, "Staff Delete Measure", "","Delete the current measure in this staff, leaving the staff short",G_CALLBACK (deletemeasure_cb)},
{"DeleteMeasureAllStaffs", NULL, "Delete Measure All Staffs", "","Delete the current measure in all staffs",G_CALLBACK (deletemeasureallstaffs_cb)},
{"ShrinkMeasures", NULL, "Shrink Measure", "","No Tooltip yet",G_CALLBACK (adjust_measure_less_width_key_cb)},
{"WidenMeasures", NULL, "Widen Measures", "","No Tooltip yet",G_CALLBACK (adjust_measure_more_width_key_cb)},
{"ShorterStaffs", NULL, "Shorter Staffs", "","No Tooltip yet",G_CALLBACK (adjust_staff_less_height_key_cb)},
{"TallerStaffs", NULL, "Taller Staffs", "","No Tooltip yet",G_CALLBACK (adjust_staff_more_height_key_cb)},
{"InsertTrebleClef", NULL, "New Treble Clef", "","No Tooltip yet",G_CALLBACK (newcleftreble_cb)},
{"InsertBassClef", NULL, "New Bass Clef", "","No Tooltip yet",G_CALLBACK (newclefbass_cb)},
{"Insertg8clef", NULL, "New G8 Clef", "","No Tooltip yet",G_CALLBACK (newclefg8_cb)},
{"InsertAltoClef", NULL, "New Alto Clef", "","No Tooltip yet",G_CALLBACK (newclefalto_cb)},
{"InsertTenorClef", NULL, "New Tenor Clef", "","No Tooltip yet",G_CALLBACK (newcleftenor_cb)},
{"InsertSopranoClef", NULL, "New Soprano Clef", "","No Tooltip yet",G_CALLBACK (newclefsoprano_cb)},
{"SetInitialTrebleClef", NULL, "Set Treble Clef", "","No Tooltip yet",G_CALLBACK (setcleftreble_cb)},
{"SetInitialBassClef", NULL, "Set Bass Clef", "","No Tooltip yet",G_CALLBACK (setclefbass_cb)},
{"SetInitialg8clef", NULL, "Set G8 Clef", "","No Tooltip yet",G_CALLBACK (setclefg8_cb)},
{"SetInitialAltoClef", NULL, "Set Alto Clef", "","No Tooltip yet",G_CALLBACK (setclefalto_cb)},
{"SetInitialTenorClef", NULL, "Set Tenor Clef", "","No Tooltip yet",G_CALLBACK (setcleftenor_cb)},
{"SetInitialSopranoClef", NULL, "Set Soprano Clef", "","No Tooltip yet",G_CALLBACK (setclefsoprano_cb)},
{"Insert22Time", NULL, "Insert 2/2 Time", "","No Tooltip yet",G_CALLBACK (newtimesig22_cb)},
{"Insert32Time", NULL, "Insert 3/2 Time", "","No Tooltip yet",G_CALLBACK (newtimesig32_cb)},
{"Insert42Time", NULL, "Insert 4/2 Time", "","No Tooltip yet",G_CALLBACK (newtimesig42_cb)},
{"Insert44Time", NULL, "Insert 4/4 Time", "","No Tooltip yet",G_CALLBACK (newtimesig44_cb)},
{"Insert34Time", NULL, "Insert 3/4 Time", "","No Tooltip yet",G_CALLBACK (newtimesig34_cb)},
{"Insert24Time", NULL, "Insert 2/4 Time", "","No Tooltip yet",G_CALLBACK (newtimesig24_cb)},
{"Insert64Time", NULL, "Insert 6/4 Time", "","No Tooltip yet",G_CALLBACK (newtimesig64_cb)},
{"Insert38Time", NULL, "Insert 3/8 Time", "","No Tooltip yet",G_CALLBACK (newtimesig38_cb)},
{"Insert68Time", NULL, "Insert 6/8 Time", "","No Tooltip yet",G_CALLBACK (newtimesig68_cb)},
{"Insert128Time", NULL, "Insert 12/8 Time", "","No Tooltip yet",G_CALLBACK (newtimesig128_cb)},
{"Insert98Time", NULL, "Insert 9/8 Time", "","No Tooltip yet",G_CALLBACK (newtimesig98_cb)},
{"Set22Time", NULL, "Set 2/2 Time", "","No Tooltip yet",G_CALLBACK (settimesig22_cb)},
{"Set32Time", NULL, "Set 3/2 Time", "","No Tooltip yet",G_CALLBACK (settimesig32_cb)},
{"Set42Time", NULL, "Set 4/2 Time", "","No Tooltip yet",G_CALLBACK (settimesig42_cb)},
{"Set44Time", NULL, "Set 4/4 Time", "","No Tooltip yet",G_CALLBACK (settimesig44_cb)},
{"Set34Time", NULL, "Set 3/4 Time", "","No Tooltip yet",G_CALLBACK (settimesig34_cb)},
{"Set24Time", NULL, "Set 2/4 Time", "","No Tooltip yet",G_CALLBACK (settimesig24_cb)},
{"Set64Time", NULL, "Set 6/4 Time", "","No Tooltip yet",G_CALLBACK (settimesig64_cb)},
{"Set38Time", NULL, "Set 3/8 Time", "","No Tooltip yet",G_CALLBACK (settimesig38_cb)},
{"Set68Time", NULL, "Set 6/8 Time", "","No Tooltip yet",G_CALLBACK (settimesig68_cb)},
{"Set128Time", NULL, "Set 12/8 Time", "","No Tooltip yet",G_CALLBACK (settimesig128_cb)},
{"Set98Time", NULL, "Set 9/8 Time", "","No Tooltip yet",G_CALLBACK (settimesig98_cb)},
{"InsertCmaj", NULL, "Insert Cmaj", "","No Tooltip yet",G_CALLBACK (newkeysigcmaj_cb)},
{"InsertGmaj", NULL, "Insert Gmaj", "","No Tooltip yet",G_CALLBACK (newkeysiggmaj_cb)},
{"InsertDmaj", NULL, "Insert Dmaj", "","No Tooltip yet",G_CALLBACK (newkeysigdmaj_cb)},
{"InsertAmaj", NULL, "Insert Amaj", "","No Tooltip yet",G_CALLBACK (newkeysigamaj_cb)},
{"InsertEmaj", NULL, "Insert Emaj", "","No Tooltip yet",G_CALLBACK (newkeysigemaj_cb)},
{"InsertBmaj", NULL, "Insert Bmaj", "","No Tooltip yet",G_CALLBACK (newkeysigbmaj_cb)},
{"InsertFSharpmaj", NULL, "Insert F# Major", "","No Tooltip yet",G_CALLBACK (newkeysigfsharpmaj_cb)},
{"InsertCSharpmaj", NULL, "Insert C# Major", "","No Tooltip yet",G_CALLBACK (newkeysigcsharpmaj_cb)},
{"InsertFmaj", NULL, "Insert F Major", "","No Tooltip yet",G_CALLBACK (newkeysigfmaj_cb)},
{"InsertBflatmaj", NULL, "Insert Bb Major", "","No Tooltip yet",G_CALLBACK (newkeysigbflatmaj_cb)},
{"InsertEflatmaj", NULL, "Insert Eb Major", "","No Tooltip yet",G_CALLBACK (newkeysigeflatmaj_cb)},
{"InsertAflatmaj", NULL, "Insert Ab Major", "","No Tooltip yet",G_CALLBACK (newkeysigaflatmaj_cb)},
{"InsertDflatmaj", NULL, "Insert Db Major", "","No Tooltip yet",G_CALLBACK (newkeysigdflatmaj_cb)},
{"InsertGflatmaj", NULL, "Insert Gb Major", "","No Tooltip yet",G_CALLBACK (newkeysiggflatmaj_cb)},
{"InsertCflatmaj", NULL, "Insert Cb Major", "","No Tooltip yet",G_CALLBACK (newkeysigcflatmaj_cb)},
{"InsertAmin", NULL, "Insert A Minor", "","No Tooltip yet",G_CALLBACK (newkeysigamin_cb)},
{"InsertEmin", NULL, "Insert E Minor", "","No Tooltip yet",G_CALLBACK (newkeysigemin_cb)},
{"InsertBmin", NULL, "Insert B Minor", "","No Tooltip yet",G_CALLBACK (newkeysigbmin_cb)},
{"InsertFSharpmin", NULL, "Insert F# Minor", "","No Tooltip yet",G_CALLBACK (newkeysigfsharpmin_cb)},
{"InsertCSharpmin", NULL, "Insert C# Minor", "","No Tooltip yet",G_CALLBACK (newkeysigcsharpmin_cb)},
{"InsertGSharpmin", NULL, "Insert G# Minor", "","No Tooltip yet",G_CALLBACK (newkeysiggsharpmin_cb)},
{"InsertDSharpmin", NULL, "Insert D# Minor", "","No Tooltip yet",G_CALLBACK (newkeysigdsharpmin_cb)},
{"InsertASharpmin", NULL, "Insert A# Minor", "","No Tooltip yet",G_CALLBACK (newkeysigasharpmin_cb)},
{"InsertDmin", NULL, "Insert D Minor", "","No Tooltip yet",G_CALLBACK (newkeysigdmin_cb)},
{"InsertGmin", NULL, "Insert G Minor", "","No Tooltip yet",G_CALLBACK (newkeysiggmin_cb)},
{"InsertCmin", NULL, "Insert C Minor", "","No Tooltip yet",G_CALLBACK (newkeysigcmin_cb)},
{"InsertFmin", NULL, "Insert F Minor", "","No Tooltip yet",G_CALLBACK (newkeysigfmin_cb)},
{"InsertBflatmin", NULL, "Insert Bb Minor", "","No Tooltip yet",G_CALLBACK (newkeysigbflatmin_cb)},
{"InsertEflatmin", NULL, "Insert Eb Minor", "","No Tooltip yet",G_CALLBACK (newkeysigeflatmin_cb)},
{"InsertAflatmin", NULL, "Insert Ab Minor", "","No Tooltip yet",G_CALLBACK (newkeysigaflatmin_cb)},
{"SetInitialCmaj", NULL, "Set Initial Keysig to C Major", "","No Tooltip yet",G_CALLBACK (setkeysigcmaj_cb)},
{"SetInitialGmaj", NULL, "Set Initial Keysig to G Major", "","No Tooltip yet",G_CALLBACK (setkeysiggmaj_cb)},
{"SetInitialDmaj", NULL, "Set D Major as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigdmaj_cb)},
{"SetInitialAmaj", NULL, "Set A Major as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigamaj_cb)},
{"SetInitialEmaj", NULL, "Set E Major as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigemaj_cb)},
{"SetInitialBmaj", NULL, "Set B Major as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigbmaj_cb)},
{"SetInitialFSharpmaj", NULL, "Set F# Major as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigfsharpmaj_cb)},
{"SetInitialCSharpmaj", NULL, "Set C# Major as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigcsharpmaj_cb)},
{"SetInitialFmaj", NULL, "Set F Major as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigfmaj_cb)},
{"SetInitialBflatmaj", NULL, "Set Bb Major as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigbflatmaj_cb)},
{"SetInitialEflatmaj", NULL, "Set Eb Major as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigeflatmaj_cb)},
{"SetInitialAflatmaj", NULL, "Set Ab Major as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigaflatmaj_cb)},
{"SetInitialDflatmaj", NULL, "Set Db Major as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigdflatmaj_cb)},
{"SetInitialGflatmaj", NULL, "Set Gb Major as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysiggflatmaj_cb)},
{"SetInitialCflatmaj", NULL, "Set Cb Major as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigcflatmaj_cb)},
{"SetInitialAmin", NULL, "Set A Minor as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigamin_cb)},
{"SetInitialEmin", NULL, "Set E Minor as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigemin_cb)},
{"SetInitialBmin", NULL, "Set B Minor as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigbmin_cb)},
{"SetInitialFSharpmin", NULL, "Set F# Minor as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigfsharpmin_cb)},
{"SetInitialCSharpmin", NULL, "Set C# Minor as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigcsharpmin_cb)},
{"SetInitialGSharpmin", NULL, "Set G# Minor as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysiggsharpmin_cb)},
{"SetInitialDSharpmin", NULL, "Set D# Minor as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigdsharpmin_cb)},
{"SetInitialASharpmin", NULL, "Set A# Minor as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigasharpmin_cb)},
{"SetInitialDmin", NULL, "Set D Minor as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigdmin_cb)},
{"SetInitialGmin", NULL, "Set G Minor as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysiggmin_cb)},
{"SetInitialCmin", NULL, "Set C Minor as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigcmin_cb)},
{"SetInitialFmin", NULL, "Set F Minor as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigfmin_cb)},
{"SetInitialBflatmin", NULL, "Set Bb Minor as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigbflatmin_cb)},
{"SetInitialEflatmin", NULL, "Set Eb Minor as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigeflatmin_cb)},
{"SetInitialAflatmin", NULL, "Set Ab Minor as Initial Keysig", "","No Tooltip yet",G_CALLBACK (setkeysigaflatmin_cb)},
{"SetMark", NULL, "Set Mark", "","Sets the start point for a selection,\nthe end point of the selection is unaltered",G_CALLBACK (set_mark_cb)},
{"UnsetMark", NULL, "Unset Mark", "","Gets rid of the selection.",G_CALLBACK (unset_mark_cb)},
{"SetPoint", NULL, "Set Point", "","Extends the selection to the current cursor position",G_CALLBACK (set_point_cb)},
{"ToggleBeginSlur", NULL, "Begin Slur", "","Insert/delete begin slur on this note",G_CALLBACK (toggle_begin_slur_cb)},
{"ToggleEndSlur", NULL, "End Slur", "","Insert/delete end slur on this note",G_CALLBACK (toggle_end_slur_cb)},
{"ToggleStartCrescendo", NULL, "Start Crescendo", "","No Tooltip yet",G_CALLBACK (toggle_start_crescendo_cb)},
{"ToggleEndCrescendo", NULL, "End Crescendo", "","No Tooltip yet",G_CALLBACK (toggle_end_crescendo_cb)},
{"ToggleStartDiminuendo", NULL, "Start Diminuendo", "","No Tooltip yet",G_CALLBACK (toggle_start_diminuendo_cb)},
{"ToggleEndDiminuendo", NULL, "End Diminuendo", "","No Tooltip yet",G_CALLBACK (toggle_end_diminuendo_cb)},
{"SetGrace", NULL, "Set Grace", "","No Tooltip yet",G_CALLBACK (set_grace_cb)},
{"ToggleGrace", NULL, "Grace Note Off/On", "","Makes the note at the cursor a grace note, if it is one, makes it normal",G_CALLBACK (toggle_grace)},
{"ForceCaution", NULL, "Force Cautionary Accidental", "","No Tooltip yet",G_CALLBACK (force_cautionary_cb)},
{"ChangePitch", NULL, "Change Pitch", "","No Tooltip yet",G_CALLBACK (change_pitch_cb)},
{"InsertRhythm", NULL, "Insert Snippet", "","No Tooltip yet",G_CALLBACK (insert_rhythm_pattern_cb)},
{"NextRhythm", NULL, "Next Snippet", "","Make next snippet\nthe current snippet.\nNotes entered will follow the rhythmic pattern of this snippet",G_CALLBACK (nextrhythm_cb)},
{"AppendMeasureAllStaffs", NULL, "Append Measure All Staffs", "","Appends a blank measure to every staff in this movement",G_CALLBACK (append_measure_score_cb)},
{"ExecuteScheme", NULL, "Execute Scheme", "","Execute the scheme code from the scripting window",G_CALLBACK (execute_scheme)},
{"FileMenu", NULL, "File", "","Menu:\nCreating, saving, loading, exporting, importing, typesetting and printing musical scores"},
{"Bookmarks", NULL, "Bookmarks", "","Creating, saving places in musical scores"},
{"ModeMenu", NULL, "Mode", "","Different keyboard and MIDI entry modes"},
{"EditMenu", NULL, "Edit", "","Menu:\nGeneral editing commands"},
{"ViewMenu", NULL, "View", "","Menu:\nControl which tools are to be shown"},
{"StaffMenu", NULL, "Staffs/Voices", "","Menu:\nCommands that manipulate Staffs and voices on staffs.\nCommands that apply to the current staff."},
{"Voices", NULL, "Voices", "","Menu:\nThings that manipulate Voices, that is separately stemmed and beamed parts that will be typeset on the same staff.\nKeep them displayed on separate staffs for editing purposes at least.\nThe typesetter will amalgamate them for you.\nSee also commands for hiding rests and moving notes and rests to avoid collisions in the Notes/Rests menu"},
{"MovementMenu", NULL, "Movements", "","Menu:\nCommands that apply to the current movement in your score.\nMovements in a score could be songs in a song book etc\nAny piece of continuous music."},
{"HelpMenu", NULL, "Help", "","Menu:\nInformation and help for Denemo"},
{"InputMenu", NULL, "Input", "","Menu:\nChoose whether to get input audio (e.g. microphone) or MIDI controller (e.g. MIDI keyboard)\nor just from the pc keyboard."},
{"SharpenEnharmonicSet", NULL, "Shift Accidentals Sharpwise", "","Shifts the set of accidentals one step sharper",G_CALLBACK (set_sharper)},
{"FlattenEnharmonicSet", NULL, "Shift Accidentals Flatwise", "","Shifts the set of accidentals one step flatter",G_CALLBACK (set_flatter)},
{"PlaybackMenu", NULL, "Playback", "","Music playback. Music is played between the playback start (green bar) and playback end (red bar).\nThere are playback controls (See View->Playback Controls) which make it easy to set and re-set these and also to loop-play, choose the temperament to play in etc. Or use items in this menu to play the whole piece of from the cursor to the end."},
{"New", GTK_STOCK_NEW, "Empty Score", "","Start a new musical score",G_CALLBACK (file_newwrapper)},
{"NewScore", GTK_STOCK_NEW, "New", "","Start a new musical score for a named instrument/voice.",G_CALLBACK (new_score_cb)},
{"OpenMenu", GTK_STOCK_OPEN, "Open", "","Menu:\nOpen a previously edited score, or a source file for transcription"},
{"Open", GTK_STOCK_OPEN, "Open", "","Open a file containing a music score for editing",G_CALLBACK (file_open_with_check)},
{"ImportLilypond", GTK_STOCK_OPEN, "Import Lilypond", "","Import a Lilypond file",G_CALLBACK (file_import_lilypond_with_check)},
{"ImportMidi", GTK_STOCK_OPEN, "Import Midi", "","Import a Midi file",G_CALLBACK (file_import_midi_with_check)},
{"ImportMusicXml", GTK_STOCK_OPEN, "Import MusicXml", "","Import a MusicXml file",G_CALLBACK (file_import_musicxml_with_check)},
{"AddStaffs", GTK_STOCK_OPEN, "Add Staffs", "","Add staffs from a Denemo file",G_CALLBACK (file_add_staffs)},
{"AddMovements", GTK_STOCK_OPEN, "Add Movement", "","Add movements from a Denemo file",G_CALLBACK (file_add_movements)},
{"MovementProps", GTK_STOCK_PROPERTIES, "Change Properties", "","Change properties of this movement",G_CALLBACK (movement_props_dialog)},
{"OpenNewWindow", GTK_STOCK_OPEN, "Open In New", "","Open a file containing a music score for editing in a separate working area (tab",G_CALLBACK (openinnew)},
{"Save", GTK_STOCK_SAVE, "Save", "","Save the score. The score is saved to disk in XML format.",G_CALLBACK (file_savewrapper)},
{"SaveAs", GTK_STOCK_SAVE_AS, "Save As", "","Save the score under a new name",G_CALLBACK (file_saveaswrapper)},
{"SaveCopy", GTK_STOCK_SAVE_AS, "Create Copy", "","Save a copy of the score",G_CALLBACK (file_copy_save)},
{"OpenTemplate", GTK_STOCK_OPEN, "Open Template", "","Start a new score from a built-in template file",G_CALLBACK (system_template_open_with_check)},
{"OpenExample", GTK_STOCK_OPEN, "Open Example", "","Start a new score from a built-in example",G_CALLBACK (system_example_open_with_check)},
{"OpenMyTemplate", GTK_STOCK_OPEN, "Open Custom Template", "","Start a new score from one of your own template files",G_CALLBACK (local_template_open_with_check)},
{"SaveTemplate", GTK_STOCK_SAVE_AS, "Save Template", "","Save the score as a template for re-use as a starting point for new scores",G_CALLBACK (template_save)},
{"NewWindow", NULL, "New Tab", "","Create working area (tab with an empty score in it)",G_CALLBACK (newview)},
{"InsertMovementBefore", NULL, "Insert Movement Before", "","Insert a new movement before the current one",G_CALLBACK (insert_movement_before)},
{"InsertMovementAfter", NULL, "Insert Movement After", "","Insert a new movement after the current one",G_CALLBACK (insert_movement_after)},
{"NewMovement", NULL, "New Movement", "","Create a new movement, usign any default template",G_CALLBACK (append_new_movement)},
{"SaveMenu", GTK_STOCK_SAVE, "Save", "","Menu:\nSave the score\nBesides saving for later you can save the score for use as a template for future works, or save under a new name etc\nThe score is saved to disk in XML format."},
{"SaveParts", GTK_STOCK_SAVE_AS, "Save Parts", "","Save Parts: each staff becomes a file in lilypond format",G_CALLBACK (file_savepartswrapper)},
{"ExportMUDELA", GTK_STOCK_SAVE_AS, "Export Lilypond", "","Export the score as a lilypond file",G_CALLBACK (export_mudela_action)},
{"ExportPDF", GTK_STOCK_SAVE_AS, "Export PDF", "","Export the score as a PDF document file",G_CALLBACK (export_pdf_action)},
{"ExportPNG", GTK_STOCK_SAVE_AS, "Export PNG", "","Export the score as a PNG image file",G_CALLBACK (export_png_action)},
{"ExportMIDI", GTK_STOCK_SAVE_AS, "Export MIDI", "","Export the score as a MIDI file",G_CALLBACK (export_midi_action)},
{"PrintMenu", GTK_STOCK_PRINT, "Print", "","Menu:\nCommands for typesetting and then printing the music.\nA part, movement, full score or various reduced scores can be typeset.\n See also the Score Layout view for further options."},
{"PrintPreview", GTK_STOCK_PRINT_PREVIEW, "Print Preview", "","Displays the final finished score in your pd viewer",G_CALLBACK (printpreview_cb)},
{"PrintView", GTK_STOCK_PRINT_PREVIEW, "Print Preview", "","Typesets the score\nIf you have a score layout selected it will use that\notherwise all movements staffs and lyrics are typeset by default.\nBe patient! It takes time to create a beautifully laid out score.\nOnce complete you can view and then send to your printer or to a file as a .pdf document.",G_CALLBACK (show_print_view)},
{"PrintSelection", GTK_STOCK_PRINT, "Print Selection", "","Displays selected music from score in your pdf viewer",G_CALLBACK (printselection_cb)},
{"PrintExcerptPreview", GTK_STOCK_PRINT_PREVIEW, "Print Excerpt", "","Displays a musical excerpt in your image viewer",G_CALLBACK (printexcerptpreview_cb)},
{"PrintMovement", GTK_STOCK_PRINT, "Print Movement", "","Typesets the current movement and opens a print dialog",G_CALLBACK (printmovement_cb)},
{"Print", GTK_STOCK_PRINT, "Print", "","Typesets the score using LilyPond and opens a print dialog",G_CALLBACK (printall_cb)},
{"PrintPart", GTK_STOCK_PRINT, "Print Part", "","Typesets the current part (the one containing the cursor).",G_CALLBACK (printpart_cb)},
{"Close", GTK_STOCK_CLOSE, "Close Score", "","Close the current score. Other scores (tabs) will stay open",G_CALLBACK (close_gui_with_check)},
{"Quit", GTK_STOCK_QUIT, "Quit", "","Quit the Denemo program - closes tabs one at a time.",G_CALLBACK (closewrapper)},
{"Undo", GTK_STOCK_UNDO, "Undo", "","Undoes one (more) step of your edits to the current score.",G_CALLBACK (undowrapper)},
{"Redo", GTK_STOCK_REDO, "Redo", "","Redoes the next of the steps you have Undone",G_CALLBACK (redowrapper)},
{"Select", NULL, "Select", "","Selecting stretches of notes"},
{"ExtendSelect", NULL, "Extend Selection", "","Extend the selection"},
{"Copy", GTK_STOCK_COPY, "Copy", "","Copy the music selected to the Denemo clipboard",G_CALLBACK (copywrapper)},
{"Cut", GTK_STOCK_CUT, "Cut", "","Cut the music selected to the Denemo clipboard",G_CALLBACK (cutwrapper)},
{"Paste", GTK_STOCK_PASTE, "Paste", "","Paste the Denemo clipboard into the score where the cursor is positioned",G_CALLBACK (pastewrapper)},
{"PasteClipboard", GTK_STOCK_PASTE, "Paste LilyPond notes", "","Paste LilyPond notes from the text clipboard\nThis will import music written as LilyPond syntax\nYou open the LilyPond file in a texteditor, copy the stretch of notes (control-c command in your texteditor usually) and then use this command.",G_CALLBACK (paste_clipboard)},
{"ScoreProperties", GTK_STOCK_PROPERTIES, "Score Properties", "","Change some of the properties of the current score. This will start up a dialog window",G_CALLBACK (score_properties_dialog)},
{"SaveSelection", NULL, "Save Selection", "","Save the selected music. Not sure if this is working",G_CALLBACK (saveselwrapper)},
{"Preferences", GTK_STOCK_PREFERENCES, "Change Preferences", "","Set and save your preferences for how Denemo operates on startup.\nAdvanced users can edit .denemo-XXXX/denemorc for missing ones",G_CALLBACK (preferences_change)},
{"KeyBindings", NULL, "Customize Commands, Shortcuts ...", "","Set actions to take in response to keypresses"},
{"SaveAccels", GTK_STOCK_SAVE, "Save Command Set", "","Save the current commands and keyboard shortcuts as the default",G_CALLBACK (save_default_keymap_file_wrapper)},
{"CommandManagement", NULL, "Manage Command Set", "","View help, change and save keyboard shortcuts",G_CALLBACK (configure_keyboard_dialog)},
{"SwapStaffs", NULL, "Swap Staffs", "","Swap this staff with the one higher up.\nBe aware that if you have inserted directives to move a voice to another staff\nthese may need re-making.",G_CALLBACK (swapstaffs)},
{"SplitVoices", NULL, "Split Voices", "","Split off the next voice as a separate staff",G_CALLBACK (splitstaffs)},
{"JoinVoices", NULL, "Join Voices", "","Merge this staff as a voice on the previous staff",G_CALLBACK (joinstaffs)},
{"SwapMovements", NULL, "Swap Movements", "","Swap this movement with the one before",G_CALLBACK (swapmovements)},
{"VoiceUp", NULL, "Voice Up", "","Go to the higher numbered voice on staff, extending selection if any",G_CALLBACK (voiceup_cb)},
{"VoiceDown", NULL, "Voice Down", "","Go to the lower numbered voice on this staff, extending selection if any",G_CALLBACK (voicedown_cb)},
{"MoveToVoiceUp", NULL, "Move to Voice Up", "","Go to the higher numbered voice on staff without altering selection",G_CALLBACK (movetovoiceup_cb)},
{"MoveToVoiceDown", NULL, "Move to Voice Down", "","Go to the lower numbered voice on this staff without altering selection",G_CALLBACK (movetovoicedown_cb)},
{"AddBefore", NULL, "Add Staff Before", "","Inserts a new staff before the current staff",G_CALLBACK (newstaffbefore)},
{"AddAfter", NULL, "Add Staff After", "","Inserts/Adds a new staff after the current staff",G_CALLBACK (dnm_newstaffafter)},
{"AddInitial", NULL, "Add Initial Staff", "","Inserts a new staff at the top of the score",G_CALLBACK (newstaffinitial)},
{"AddLast", NULL, "Add Last Staff", "","Inserts a new staff at the end of the score",G_CALLBACK (newstafflast)},
{"DeleteBefore", NULL, "Delete Staff Before", "","Deletes the staff before the current staff",G_CALLBACK (delete_staff_before)},
{"DeleteStaff", NULL, "Delete Current Staff", "","Deletes the current staff",G_CALLBACK (delete_staff_current)},
{"DeleteAfter", NULL, "Delete Staff After", "","Deletes the staff after the current staff",G_CALLBACK (delete_staff_after)},
{"AddVoice", NULL, "Add Voice", "","Adds a new voice (part), to the current staff. It is tricky to switch between the voices. Suggest to use merge staffs",G_CALLBACK (dnm_newstaffvoice)},
{"StaffProperties", GTK_STOCK_PROPERTIES, "Staff Properties", "","Change the properties of the current staff",G_CALLBACK (staff_properties_change_cb)},
{"InsertMenu", NULL, "InsertMenu", "","Insert"},
{"Clef", NULL, "Clef", "","Menu:\nClef"},
{"InitialClef", NULL, "Initial Clef", "","Change the initial clef of the current staff",G_CALLBACK (clef_change_initial)},
{"InsertClef", NULL, "Clef Change", "","Insert/Edit a change of clef at the cursor",G_CALLBACK (clef_change_insert)},
{"Key", NULL, "Keys", "","Menu:\nSet the key signature or insert a change of key signature\nSharpen or flatten a key signature or\nhide it on printing"},
{"InitialKey", NULL, "Initial Key", "","Set the initial key signature of the current staff",G_CALLBACK (key_change_initial)},
{"InsertKey", NULL, "Key Signature Change", "","Insert/Edit a key change at the cursor position",G_CALLBACK (key_change_insert)},
{"TimeSig", NULL, "Time Signatures", "","Menu:\nSet the time signature, insert or edit a change of timesignature,\nhide a time signature on printing.\nControl whether to use numerical or traditional styles."},
{"InitialTimeSig", NULL, "Inital Time Signature", "","Set the initial time signature of the current staff",G_CALLBACK (timesig_change_initial)},
{"InsertTimeSig", NULL, "Time Signature Change", "","Edit/Insert a time signature change for the current measure",G_CALLBACK (timesig_change_insert)},
{"ChangeNotehead", NULL, "Set Notehead", "","Change the type of notehead for the current note",G_CALLBACK (set_notehead)},
{"InsertStem", NULL, "Auto Stemming", "","Inserts a stem neutral object. After this automatic stem directions are active. You can click on this tag and use Sharpen/StemUp etc commands to change stem direction",G_CALLBACK (stem_directive_insert)},
{"AddVerse", NULL, "Add Lyric Verse", "","Add a verse of lyrics",G_CALLBACK (add_verse)},
{"DeleteVerse", NULL, "Delete Verse", "","Deletes current verse of lyrics from current voice",G_CALLBACK (delete_verse)},
{"EditFiguredBass", NULL, "Insert/Edit Figured Bass", "","Add a bass figure to the current note. Use | sign to split the duration of a note so as to have multiple figures on one note. See Lilypond docs for other notation",G_CALLBACK (figure_insert)},
{"DeleteFiguredBass", NULL, "Delete Figures", "","Delete the figured bass on the current staff",G_CALLBACK (delete_figured_bass)},
{"HideFiguredBass", NULL, "Hide Figures (Print)", "","Hide the figured bass on the current staff on printing",G_CALLBACK (hide_figured_bass)},
{"ShowFiguredBass", NULL, "Show Figures (Print)", "","Show the figured bass on the current staff on printing",G_CALLBACK (show_figured_bass)},
{"EditChords", NULL, "Edit Chord Symbols", "","Allows chord symbols to be added to the current note. E.G.cis:dim7 for c-sharp diminished 7th. See Lilypond docs for notation",G_CALLBACK (fakechord_insert)},
{"InsertDynamic", NULL, "Insert Dynamics", "","Inserts a dynamic marking at the cursor position",G_CALLBACK (insert_dynamic)},
{"EditObject", NULL, "Edit Object", "","Edit the object at the cursor.",G_CALLBACK (edit_object)},
{"EditDirective", NULL, "Edit Directives", "","Edit any directives attached to chord/note at cursor.",G_CALLBACK (edit_object_directive)},
{"EditStaffDirective", NULL, "Edit Staff Directives", "","Edit any directives attached to staff.",G_CALLBACK (edit_staff_directive)},
{"EditVoiceDirective", NULL, "Edit Voice Directives", "","Edit any directives attached to voice.",G_CALLBACK (edit_voice_directive)},
{"EditScoreDirective", NULL, "Edit Score Directives", "","Edit any directives attached to score.",G_CALLBACK (edit_score_directive)},
{"EditMovementDirective", NULL, "Edit Movement Directives", "","Edit any directives attached to movement.",G_CALLBACK (edit_movement_directive)},
{"EditClefDirective", NULL, "Edit Clef Directives", "","Edit any directives attached to clef.",G_CALLBACK (edit_clef_directive)},
{"EditTimesigDirective", NULL, "Edit Time Signature Directives", "","Edit any directives attached to time signature.",G_CALLBACK (edit_timesig_directive)},
{"EditKeysigDirective", NULL, "Edit Key Signature Directives", "","Edit any directives attached to key signature.",G_CALLBACK (edit_keysig_directive)},
{"DeleteDirective", NULL, "Delete a Directive", "","Delete a directive attached to chord/note at cursor.",G_CALLBACK (delete_chord_or_note_directive)},
{"AttachLilyToNote", NULL, "Attach Lilypond to Note", "","Attach or edit attached LilyPond text to the note at the cursor. This can be used for guitar fingerings, cautionary accidentals and much more. See LilyPond documentation.",G_CALLBACK (note_directive)},
{"AttachLilyToChord", NULL, "Attach Lilypond to Chord", "","Attach or edit attached LilyPond text to the chord at the cursor. This can be used for attaching and placing text and much more. See LilyPond documentation.",G_CALLBACK (chord_directive)},
{"InsertBarline", NULL, "Insert Barline", "","Inserts specialized barline at the cursor position. Mostly not working",G_CALLBACK (insert_barline)},
{"NavigationMenu", NULL, "Navigation", "","Moving the cursor within the current movement, setting bookmarks, seeking things in the score"},
{"GoToMeasure", NULL, "Go to Measure", "","Opens a dialog for going to a numbered measure",G_CALLBACK (tomeasurenum)},
{"GoToBeginning", GTK_STOCK_GOTO_FIRST, "Go to Beginning", "","Cursor to start of staff/voice, extending selection if any",G_CALLBACK (tohome)},
{"GoToEnd", GTK_STOCK_GOTO_LAST, "Go to End", "","Cursor to end of staff/voice, extending selection if any",G_CALLBACK (toend)},
{"MoveToBeginning", GTK_STOCK_GOTO_FIRST, "Move to Staff/Voice Beginning", "","Cursor to start of staff/voice, without extending selection if any",G_CALLBACK (movetostart)},
{"MoveToEnd", GTK_STOCK_GOTO_LAST, "Move to Staff/Voice End", "","Cursor to end of staff/voice, without extending selection if any",G_CALLBACK (movetoend)},
{"NextMovement", NULL, "Next Movement", "","Go to the next movement",G_CALLBACK (next_movement)},
{"PreviousMovement", NULL, "Previous Movement", "","Go to the previous movement",G_CALLBACK (prev_movement)},
{"DeleteMovement", NULL, "Delete Movement", "","Delete the current movement",G_CALLBACK (delete_movement)},
{"Play", GTK_STOCK_MEDIA_PLAY, "Play", "","Play",G_CALLBACK (ext_midi_playback)},
{"Stop", GTK_STOCK_MEDIA_STOP, "Stop", "","Stop",G_CALLBACK (stop_midi_playback)},
{"PlaybackProperties", GTK_STOCK_PROPERTIES, "Playback Properties", "","Allows you to specify properties used in playing back (midi)",G_CALLBACK (playback_properties_change)},
{"Help", NULL, "Browse Manual", "","Opens a browser on the user manual",G_CALLBACK (browse_manual)},
{"About", NULL, "About", "","Gives the version number etc of this program",G_CALLBACK (about)},
{"MoreMenu", NULL, "More", "","Allows choosing extra commands/menu items from disk"},
{"MoreCommands", NULL, "More Commands", "","Allows choosing standard extra commands/menu items",G_CALLBACK (morecommands)},
{"MyCommands", NULL, "My Commands", "","Allows choosing extra commands/menu items from your own collection of extras",G_CALLBACK (mycommands)},
{"FetchCommands", NULL, "Update Commands from Internet", "","Refreshes the set of commands available from Denemo.org.\nUse More Commands after this has finished",G_CALLBACK (fetchcommands)},
{"OpenRecent", GTK_STOCK_OPEN, "Open Recent", "","Open previously used files"},
{"Import", GTK_STOCK_OPEN, "Import File", "","Menu:\nImport the supported file formats"},
{"Export", NULL, "Export As", "","Menu:\nExport the music to another file format.\nThese file formats will not support all the features of the score."},
{"ToggleEdit", NULL, "Toggle Edit Mode", "","Toggle between current mode and edit mode",G_CALLBACK (toggle_edit_mode)},
{"ToggleRest", NULL, "Toggle Rest Mode", "","Toggle between note entry and rest entry",G_CALLBACK (toggle_rest_mode)},
{"ToggleRhythm", NULL, "Toggle Audible Feedback", "","Toggle audible feedback on/off",G_CALLBACK (toggle_rhythm_mode)},
{"ClearOverlay", NULL, "Clear Overlay", "","Clear the list of pitches that overlay the notes",G_CALLBACK (clear_overlay)},
{"CreateRhythm", NULL, "Create Snippet", "","Copy selection as music snippet or rhythm pattern for notes to follow as they are entered",G_CALLBACK (create_rhythm_cb)},
{"DeleteRhythm", NULL, "Delete Snippet", "","Delete the selected music snippet/rhythm pattern",G_CALLBACK (delete_rhythm_cb)},
{"ClassicModeNote", NULL, "Classic Mode", "","Menu:\nMoving the cursor and inserting notes or rests there"},
{"SelectNote", NULL, "Select Note", "","Menu:\nMoving the cursor to the nearest ..."},
{"InsertModeNote", NULL, "Insert", "","Menu:\nCommands for notes: inserting, deleting, etc."},
{"StemControl", NULL, "Stem Direction", "","Menu:\nCommands to control the stem up/down"},
{"NoteheadControl", NULL, "Notehead Types", "","Menu:\nDifferent types of notehead"},
{"TiedNotes", NULL, "Tied Notes", "","Menu:\nCreating Tied Notes\nDo not confuse with slurs!!"},
{"DisplayEffects", NULL, "Display Effects", "","Menu:\nEffects that are only for the Denemo display,\nnot affecting the printed page."},
{"Articulations", NULL, "Articulations", "","Menu:\nArticulations"},
{"Slurs", NULL, "Slurs", "","Menu:\nSlurs\nDo not confuse with ties which in print can look the same\nSlurs have little ticks at the end in the Denemo display."},
{"GraceNotes", NULL, "Grace Notes", "","Menu:\nGrace Notes"},
{"Score", NULL, "Score", "","Menu:\nCommands that affect the entire piece of music across all movements."},
{"Instruments", NULL, "Instruments", "","Menu:\nInstruments"},
{"Markings", NULL, "Markings", "","Menu:\nMarkings to be applied to the note at the cursor.\nIncludes articulations, string numbers, chord symbols, arbitrary text ..."},
{"TextMarks", NULL, "Text/Symbol", "","Menu:\nText and symbol Markings"},
{"Strings", NULL, "Strings", "","Menu:\nCommands for Stringed Instruments"},
{"InsertNote", NULL, "Note Insertion", "","Menu:\nInserting the note ..."},
{"Lyrics", NULL, "Lyrics", "","Menu:\nLyrics consist of syllables which are placed under or above the notes of the voice they are attached to\nYou can use the hyphen - and underscore _ to affect the placement\nas well as slurs placed over notes. Spaces, tabs or new lines can serve to separate the syllables, it makes no difference which."},
{"Directives", NULL, "Directives", "","Menu:\nDirectives tell Denemo to do something special with the object they are attached to\nor at the point in the music they are placed\nThis can be directing the typesetter to do something special\nor issuing MIDI instructions to alter the playback\nor even take an action, such as linking to source manuscript when clicked"},
{"Navigation", NULL, "Navigation", "","Menu:\nMoving the Denemo cursor around the piece"},
{"NoteEntry", NULL, "Insert Note", "","Menu:\nEntering notes"},
{"Articulation", NULL, "Articulation", "","Menu:\nMarks placed on notes and chords"},
{"Edit", NULL, "Edit", "","Menu:\nEditing"},
{"Measure", NULL, "Measure", "","Menu:\nManipulating measures\nCommands that apply to the current measure"},
{"Staff", NULL, "Staff", "","Menu:\nCommands for staffs"},
{"Playback", NULL, "Playback", "","Menu:\nPlaying the music via the computer's soundcard.\nAll or part of the piece can be played controlled by the green and red markers (start and stop playing)\nUse the Playback Controls (from the View menu) to set/reset these and to loop play while editing the score"},
{"SelectDuration", NULL, "Select Duration", "","Menu:\nChanging the prevailing duration or rhythm pattern"},
{"EditModeNote", NULL, "Append/Edit", "","Menu:\nAppending, Changing, and deleting notes"},
{"EditNote", NULL, "Append/Edit Note", "","Menu:\nAppending or Editing notes"},
{"EditDuration", NULL, "Append/Insert Duration", "","Menu:\nAppending or Editing durations"},
{"Cursor", NULL, "Cursor", "","Menu:\nMoving the cursor"},
{"CursorToNote", NULL, "Cursor to Note", "","Menu:\nMoving the cursor to note positions"},
{"ClefMenu", NULL, "Clefs", "","Menu:\nInsert/change clef, set initial clef, hide a clef on printing,\nor display in a different clef"},
{"ChordMenu", NULL, "Chords", "","Menu:\nCommands for creating chords. Note that chord symbols and figured bass are under markings on notes.\nSee Notes/Rests menu"},
{"ChordNoteMenu", NULL, "Add Note", "","Menu:\nAdding notes to a note or chord at the cursor to make chords"},
{"ChordSymbols", NULL, "Chords Symbols", "","Menu:\nAdding Chord Symbols over music"},
{"FiguredBass", NULL, "Figured Bass", "","Menu:\nAdding Figured Bass Figures"},
{"MeasureMenu", NULL, "Measures", "","Menu:\nMeasures: adding, deleting, navigating etc"},
{"Insert", NULL, "Insert", "","Menu:\nInserting notes, measures, staffs, keysignatures etc"},
{"InsertStaff", NULL, "Add Staff", "","Menu:\nInsert a new staff postioned relative to current staff"},
{"InsertMovement", NULL, "Insert Movement", "","Menu:\nInsert a new movement positioned relative to current movement"},
{"InsertDuration", NULL, "Insert at Cursor", "","Menu:\nInserting notes of a given duration at cursor note height"},
{"Change", NULL, "Change", "","Menu:\nChanging properties of notes, measures, staffs, keysigs etc"},
{"ModelessNote", NULL, "Notes/Durations", "","Menu:\nModeless actions on notes/rests"},
{"NotesRests", NULL, "Notes/Rests", "","Menu:\nCommands for notes/rests"},
{"RestEntry", NULL, "Rest Insertion", "","Menu:\nModeless entry of rests"},
{"Educational", NULL, "Educational", "","Menu:\nEducational Games"},
{"EditDirectivesMenu", NULL, "Edit Directive(s)", "","Menu:\nEditing directives"},
{"ChangeNote", NULL, "Change Note", "","Menu:\nChanging the note at the cursor to the nearest ..."},
{"ChangeDuration", NULL, "Edit Duration", "","Menu:\nChanges the duration of the current note"},
{"ChangeRest", NULL, "Change Rest", "","Menu:\nChanges the duration of the current rest"},
{"ExpressionMarks", NULL, "Expression Marks", "","Menu:\nDynamics, staccato, slurs, ties and other expressive marks"},
{"Dynamics", NULL, "Dynamics", "","Dynamic markings"},
{"Ornaments", NULL, "Ornaments", "","Menu:\ngrace notes etc"},
{"Other", NULL, "Other", "","Lyrics, chord symbols, figured basses etc"},
{"Others", NULL, "Others", "","Less used actions"},
{"Favorites", NULL, "Favorites", "","Customized LilyPond inserts. Store often-used inserts here labelled with what they do"},
{"AddFavorite", NULL, "Add Favorite", "","Menu:\nAdd a custom LilyPond insert to favorites menu"},
{"Tuplets", NULL, "Tuplets", "","Menu:\nEntering triplets and other tuplets"},
{"Delete", NULL, "Delete", "","Menu:\nDeleting notes, measures staffs keysigs etc"},
{"Beaming", NULL, "Beaming", "","Menu:\nBeaming - controlling which notes are beamed together"},
{"Conditional Directives", NULL, "Conditional Directives", "","Menu:\nFor making directives apply only to one score layout or be omitted from one score layout"},
{"MIDI", NULL, "MIDI Keyboard", "","Menu:\nFor affecting how MIDI in works. MIDI in refers to MIDI keyboards or other controllers connected to the computer. You may need to configure this using the Audio/MIDI section of the Preferences."},
{"Keyboard", NULL, "PC Keyboard", "","Menu:\nChoose between different ways of entering notes from the computer keyboard."},
{"Mouse", NULL, "Mouse", "","Menu:\nChoose between different ways of using the mouse."},

#define NOTE0 "\xF0\x9D\x85\x9D"

#define REST0 "\xF0\x9D\x84\xBB"
{"0", "NULL", NOTE0, NULL, "Edits the note at the cursor to have the duration "NOTE0", or \n(if appending) appends such a duration -  to be completed with a note name. Changes prevailing duration",
G_CALLBACK (Dur0)},
{"Change0", "NULL", NOTE0, NULL, "Change current note to a "NOTE0,
G_CALLBACK (ChangeDur0)},
{"ChangeRest0", NULL, "Change duration", NULL, "Change duration of current rest",
G_CALLBACK (ChangeRest0)},
{"Insert0", NULL, "Insert a "NOTE0"", NULL, "Inserts a "NOTE0" at cursor position\nSets prevailing rhythm to "NOTE0,
G_CALLBACK (InsertDur0)},
{"InsertRest0", NULL, "Insert a "REST0"rest", NULL, "Inserts a rest at cursor position\nSets prevailing rhythm to "NOTE0,
G_CALLBACK (InsertRest0)},
{"Set0", NULL, "Set Duration to "NOTE0"", NULL, "Sets prevailing rhythm to "NOTE0,
G_CALLBACK (SetDur0)},

#define NOTE1 "\xF0\x9D\x85\x9E"

#define REST1 "\xF0\x9D\x84\xBC"
{"1", "NULL", NOTE1, NULL, "Edits the note at the cursor to have the duration "NOTE1", or \n(if appending) appends such a duration -  to be completed with a note name. Changes prevailing duration",
G_CALLBACK (Dur1)},
{"Change1", "NULL", NOTE1, NULL, "Change current note to a "NOTE1,
G_CALLBACK (ChangeDur1)},
{"ChangeRest1", NULL, "Change duration", NULL, "Change duration of current rest",
G_CALLBACK (ChangeRest1)},
{"Insert1", NULL, "Insert a "NOTE1"", NULL, "Inserts a "NOTE1" at cursor position\nSets prevailing rhythm to "NOTE1,
G_CALLBACK (InsertDur1)},
{"InsertRest1", NULL, "Insert a "REST1"rest", NULL, "Inserts a rest at cursor position\nSets prevailing rhythm to "NOTE1,
G_CALLBACK (InsertRest1)},
{"Set1", NULL, "Set Duration to "NOTE1"", NULL, "Sets prevailing rhythm to "NOTE1,
G_CALLBACK (SetDur1)},

#define NOTE2 "\xF0\x9D\x85\x9F"

#define REST2 "\xF0\x9D\x84\xBD"
{"2", "NULL", NOTE2, NULL, "Edits the note at the cursor to have the duration "NOTE2", or \n(if appending) appends such a duration -  to be completed with a note name. Changes prevailing duration",
G_CALLBACK (Dur2)},
{"Change2", "NULL", NOTE2, NULL, "Change current note to a "NOTE2,
G_CALLBACK (ChangeDur2)},
{"ChangeRest2", NULL, "Change duration", NULL, "Change duration of current rest",
G_CALLBACK (ChangeRest2)},
{"Insert2", NULL, "Insert a "NOTE2"", NULL, "Inserts a "NOTE2" at cursor position\nSets prevailing rhythm to "NOTE2,
G_CALLBACK (InsertDur2)},
{"InsertRest2", NULL, "Insert a "REST2"rest", NULL, "Inserts a rest at cursor position\nSets prevailing rhythm to "NOTE2,
G_CALLBACK (InsertRest2)},
{"Set2", NULL, "Set Duration to "NOTE2"", NULL, "Sets prevailing rhythm to "NOTE2,
G_CALLBACK (SetDur2)},

#define NOTE3 "\xF0\x9D\x85\xA0"

#define REST3 "\xF0\x9D\x84\xBE"
{"3", "NULL", NOTE3, NULL, "Edits the note at the cursor to have the duration "NOTE3", or \n(if appending) appends such a duration -  to be completed with a note name. Changes prevailing duration",
G_CALLBACK (Dur3)},
{"Change3", "NULL", NOTE3, NULL, "Change current note to a "NOTE3,
G_CALLBACK (ChangeDur3)},
{"ChangeRest3", NULL, "Change duration", NULL, "Change duration of current rest",
G_CALLBACK (ChangeRest3)},
{"Insert3", NULL, "Insert a "NOTE3"", NULL, "Inserts a "NOTE3" at cursor position\nSets prevailing rhythm to "NOTE3,
G_CALLBACK (InsertDur3)},
{"InsertRest3", NULL, "Insert a "REST3"rest", NULL, "Inserts a rest at cursor position\nSets prevailing rhythm to "NOTE3,
G_CALLBACK (InsertRest3)},
{"Set3", NULL, "Set Duration to "NOTE3"", NULL, "Sets prevailing rhythm to "NOTE3,
G_CALLBACK (SetDur3)},

#define NOTE4 "\xF0\x9D\x85\xA1"

#define REST4 "\xF0\x9D\x84\xBF"
{"4", "NULL", NOTE4, NULL, "Edits the note at the cursor to have the duration "NOTE4", or \n(if appending) appends such a duration -  to be completed with a note name. Changes prevailing duration",
G_CALLBACK (Dur4)},
{"Change4", "NULL", NOTE4, NULL, "Change current note to a "NOTE4,
G_CALLBACK (ChangeDur4)},
{"ChangeRest4", NULL, "Change duration", NULL, "Change duration of current rest",
G_CALLBACK (ChangeRest4)},
{"Insert4", NULL, "Insert a "NOTE4"", NULL, "Inserts a "NOTE4" at cursor position\nSets prevailing rhythm to "NOTE4,
G_CALLBACK (InsertDur4)},
{"InsertRest4", NULL, "Insert a "REST4"rest", NULL, "Inserts a rest at cursor position\nSets prevailing rhythm to "NOTE4,
G_CALLBACK (InsertRest4)},
{"Set4", NULL, "Set Duration to "NOTE4"", NULL, "Sets prevailing rhythm to "NOTE4,
G_CALLBACK (SetDur4)},

#define NOTE5 "\xF0\x9D\x85\xA2"

#define REST5 "\xF0\x9D\x85\x80"
{"5", "NULL", NOTE5, NULL, "Edits the note at the cursor to have the duration "NOTE5", or \n(if appending) appends such a duration -  to be completed with a note name. Changes prevailing duration",
G_CALLBACK (Dur5)},
{"Change5", "NULL", NOTE5, NULL, "Change current note to a "NOTE5,
G_CALLBACK (ChangeDur5)},
{"ChangeRest5", NULL, "Change duration", NULL, "Change duration of current rest",
G_CALLBACK (ChangeRest5)},
{"Insert5", NULL, "Insert a "NOTE5"", NULL, "Inserts a "NOTE5" at cursor position\nSets prevailing rhythm to "NOTE5,
G_CALLBACK (InsertDur5)},
{"InsertRest5", NULL, "Insert a "REST5"rest", NULL, "Inserts a rest at cursor position\nSets prevailing rhythm to "NOTE5,
G_CALLBACK (InsertRest5)},
{"Set5", NULL, "Set Duration to "NOTE5"", NULL, "Sets prevailing rhythm to "NOTE5,
G_CALLBACK (SetDur5)},

#define NOTE6 "\xF0\x9D\x85\xA3"

#define REST6 "\xF0\x9D\x85\x81"
{"6", "NULL", NOTE6, NULL, "Edits the note at the cursor to have the duration "NOTE6", or \n(if appending) appends such a duration -  to be completed with a note name. Changes prevailing duration",
G_CALLBACK (Dur6)},
{"Change6", "NULL", NOTE6, NULL, "Change current note to a "NOTE6,
G_CALLBACK (ChangeDur6)},
{"ChangeRest6", NULL, "Change duration", NULL, "Change duration of current rest",
G_CALLBACK (ChangeRest6)},
{"Insert6", NULL, "Insert a "NOTE6"", NULL, "Inserts a "NOTE6" at cursor position\nSets prevailing rhythm to "NOTE6,
G_CALLBACK (InsertDur6)},
{"InsertRest6", NULL, "Insert a "REST6"rest", NULL, "Inserts a rest at cursor position\nSets prevailing rhythm to "NOTE6,
G_CALLBACK (InsertRest6)},
{"Set6", NULL, "Set Duration to "NOTE6"", NULL, "Sets prevailing rhythm to "NOTE6,
G_CALLBACK (SetDur6)},

#define NOTE7 "\xF0\x9D\x85\xA4"

#define REST7 "\xF0\x9D\x85\x82"
{"7", "NULL", NOTE7, NULL, "Edits the note at the cursor to have the duration "NOTE7", or \n(if appending) appends such a duration -  to be completed with a note name. Changes prevailing duration",
G_CALLBACK (Dur7)},
{"Change7", "NULL", NOTE7, NULL, "Change current note to a "NOTE7,
G_CALLBACK (ChangeDur7)},
{"ChangeRest7", NULL, "Change duration", NULL, "Change duration of current rest",
G_CALLBACK (ChangeRest7)},
{"Insert7", NULL, "Insert a "NOTE7"", NULL, "Inserts a "NOTE7" at cursor position\nSets prevailing rhythm to "NOTE7,
G_CALLBACK (InsertDur7)},
{"InsertRest7", NULL, "Insert a "REST7"rest", NULL, "Inserts a rest at cursor position\nSets prevailing rhythm to "NOTE7,
G_CALLBACK (InsertRest7)},
{"Set7", NULL, "Set Duration to "NOTE7"", NULL, "Sets prevailing rhythm to "NOTE7,
G_CALLBACK (SetDur7)},

#define NOTE8 "\xF0\x9D\x85\xA5"

#define REST8 "\xF0\x9D\x85\x83"
{"8", "NULL", NOTE8, NULL, "Edits the note at the cursor to have the duration "NOTE8", or \n(if appending) appends such a duration -  to be completed with a note name. Changes prevailing duration",
G_CALLBACK (Dur8)},
{"Change8", "NULL", NOTE8, NULL, "Change current note to a "NOTE8,
G_CALLBACK (ChangeDur8)},
{"ChangeRest8", NULL, "Change duration", NULL, "Change duration of current rest",
G_CALLBACK (ChangeRest8)},
{"Insert8", NULL, "Insert a "NOTE8"", NULL, "Inserts a "NOTE8" at cursor position\nSets prevailing rhythm to "NOTE8,
G_CALLBACK (InsertDur8)},
{"InsertRest8", NULL, "Insert a "REST8"rest", NULL, "Inserts a rest at cursor position\nSets prevailing rhythm to "NOTE8,
G_CALLBACK (InsertRest8)},
{"Set8", NULL, "Set Duration to "NOTE8"", NULL, "Sets prevailing rhythm to "NOTE8,
G_CALLBACK (SetDur8)},
 {"InsertA", NULL, "Insert A", NULL, "Inserts note A before note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (InsertA)},
 {"AddNoteA", NULL, "Insert A After", NULL, "Inserts note A after note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (AddNoteA)},
 {"AddA", NULL, "Add A to Chord", NULL, "Adds note A to chord at cursor\nCursor determines which octave",
  G_CALLBACK (AddA)},
  {"ChangeToA", NULL, "Change current note to A", NULL, "Changes current note to the A nearest cursor or (if no current note) inserts the note A\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
   G_CALLBACK (ChangeToA)},
  {"MoveToA", NULL, "Move cursor to step A", NULL, "Moves the cursor to the A nearest cursor\nCurrent cursor position determines which octave.",
   G_CALLBACK (MoveToA)},
 {"InsertB", NULL, "Insert B", NULL, "Inserts note B before note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (InsertB)},
 {"AddNoteB", NULL, "Insert B After", NULL, "Inserts note B after note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (AddNoteB)},
 {"AddB", NULL, "Add B to Chord", NULL, "Adds note B to chord at cursor\nCursor determines which octave",
  G_CALLBACK (AddB)},
  {"ChangeToB", NULL, "Change current note to B", NULL, "Changes current note to the B nearest cursor or (if no current note) inserts the note B\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
   G_CALLBACK (ChangeToB)},
  {"MoveToB", NULL, "Move cursor to step B", NULL, "Moves the cursor to the B nearest cursor\nCurrent cursor position determines which octave.",
   G_CALLBACK (MoveToB)},
 {"InsertC", NULL, "Insert C", NULL, "Inserts note C before note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (InsertC)},
 {"AddNoteC", NULL, "Insert C After", NULL, "Inserts note C after note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (AddNoteC)},
 {"AddC", NULL, "Add C to Chord", NULL, "Adds note C to chord at cursor\nCursor determines which octave",
  G_CALLBACK (AddC)},
  {"ChangeToC", NULL, "Change current note to C", NULL, "Changes current note to the C nearest cursor or (if no current note) inserts the note C\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
   G_CALLBACK (ChangeToC)},
  {"MoveToC", NULL, "Move cursor to step C", NULL, "Moves the cursor to the C nearest cursor\nCurrent cursor position determines which octave.",
   G_CALLBACK (MoveToC)},
 {"InsertD", NULL, "Insert D", NULL, "Inserts note D before note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (InsertD)},
 {"AddNoteD", NULL, "Insert D After", NULL, "Inserts note D after note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (AddNoteD)},
 {"AddD", NULL, "Add D to Chord", NULL, "Adds note D to chord at cursor\nCursor determines which octave",
  G_CALLBACK (AddD)},
  {"ChangeToD", NULL, "Change current note to D", NULL, "Changes current note to the D nearest cursor or (if no current note) inserts the note D\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
   G_CALLBACK (ChangeToD)},
  {"MoveToD", NULL, "Move cursor to step D", NULL, "Moves the cursor to the D nearest cursor\nCurrent cursor position determines which octave.",
   G_CALLBACK (MoveToD)},
 {"InsertE", NULL, "Insert E", NULL, "Inserts note E before note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (InsertE)},
 {"AddNoteE", NULL, "Insert E After", NULL, "Inserts note E after note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (AddNoteE)},
 {"AddE", NULL, "Add E to Chord", NULL, "Adds note E to chord at cursor\nCursor determines which octave",
  G_CALLBACK (AddE)},
  {"ChangeToE", NULL, "Change current note to E", NULL, "Changes current note to the E nearest cursor or (if no current note) inserts the note E\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
   G_CALLBACK (ChangeToE)},
  {"MoveToE", NULL, "Move cursor to step E", NULL, "Moves the cursor to the E nearest cursor\nCurrent cursor position determines which octave.",
   G_CALLBACK (MoveToE)},
 {"InsertF", NULL, "Insert F", NULL, "Inserts note F before note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (InsertF)},
 {"AddNoteF", NULL, "Insert F After", NULL, "Inserts note F after note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (AddNoteF)},
 {"AddF", NULL, "Add F to Chord", NULL, "Adds note F to chord at cursor\nCursor determines which octave",
  G_CALLBACK (AddF)},
  {"ChangeToF", NULL, "Change current note to F", NULL, "Changes current note to the F nearest cursor or (if no current note) inserts the note F\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
   G_CALLBACK (ChangeToF)},
  {"MoveToF", NULL, "Move cursor to step F", NULL, "Moves the cursor to the F nearest cursor\nCurrent cursor position determines which octave.",
   G_CALLBACK (MoveToF)},
 {"InsertG", NULL, "Insert G", NULL, "Inserts note G before note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (InsertG)},
 {"AddNoteG", NULL, "Insert G After", NULL, "Inserts note G after note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (AddNoteG)},
 {"AddG", NULL, "Add G to Chord", NULL, "Adds note G to chord at cursor\nCursor determines which octave",
  G_CALLBACK (AddG)},
  {"ChangeToG", NULL, "Change current note to G", NULL, "Changes current note to the G nearest cursor or (if no current note) inserts the note G\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
   G_CALLBACK (ChangeToG)},
  {"MoveToG", NULL, "Move cursor to step G", NULL, "Moves the cursor to the G nearest cursor\nCurrent cursor position determines which octave.",
   G_CALLBACK (MoveToG)},
