;;;ToggleHarmonic
;use _instead of ^ to place harmonic below note
(ToggleChordDirective "Harmonic" LG-Harmonic "^\\flageolet" DENEMO_OVERRIDE_ABOVE)
(d-DirectivePut-chord-gy "Harmonic" 12)
;(d-DirectivePut-chord-gx "Harmonic" -2)
