(d-DirectivePut-score-postfix  "TransposeScore" "\\transpose c c' ")
(d-DirectivePut-score-display  "TransposeScorePrint" "Print out 8va up")
(d-DirectivePut-score-override  "TransposeScorePrint" DENEMO_OVERRIDE_GRAPHIC)

(d-RefreshDisplay)
