/* playbackprops.h 
 *
 * playback propertues callback function prototype 
 *
 * for Denemo, a gtk+ frontend for GNU Lilypond
 * (c) 2000-2005 Adam Tee, Matthew Hiller
 */ 

#ifndef PLAYBACKPROPS_H
#define PLAYBACKPROPS_H



static void
set_playback (GtkWidget * widget, gpointer data);
#endif