;;;EndVolta
(d-Directive-standalone "EndVolta")
(d-DirectivePut-standalone-minpixels  "EndVolta" 50)
(d-DirectivePut-standalone-postfix "EndVolta" "
\\set Score.repeatCommands = #'((volta #f))
")
(d-DirectivePut-standalone-gx  "EndVolta" 18)
(d-DirectivePut-standalone-gy  "EndVolta" -36)
(d-DirectivePut-standalone-graphic "EndVolta" "EndSecondTimeBar")
(d-MoveCursorRight)
(d-RefreshDisplay)
 