;;D.C al fine
(d-DirectivePut-chord-display "D.C.AlFine"  "D.C. al fine" )
(d-DirectivePut-chord-postfix "D.C.AlFine"  "^\\markup \\italic \\bold {D. C. al fine}")
(d-DirectivePut-chord-minpixels  "D.C.AlFine" 20)
(d-RefreshDisplay)