(ToggleDirective "staff" "postfix" "TabStaff" " \\new TabStaff <<\n"  DENEMO_OVERRIDE_LILYPOND)
(ToggleDirective "voice" "postfix" "TabStaff" "\n"  DENEMO_OVERRIDE_LILYPOND)
(ToggleDirective "clef" "postfix" "TabStaff" "{} \n"  DENEMO_OVERRIDE_LILYPOND)