;;;;;;;; NormalBeamEndings
(d-ScoreBlockProlog "
#(score-override-auto-beam-setting '(end * * 3 4) 1 4)
#(score-override-auto-beam-setting '(end * * 3 4) 2 4)
#(score-override-auto-beam-setting '(end * * 4 4) 1 4)
#(score-override-auto-beam-setting '(end * * 4 4) 2 4)
#(score-override-auto-beam-setting '(end * * 4 4) 3 4)
#(score-override-auto-beam-setting '(end 1 16 2 2) 1 4)
#(score-override-auto-beam-setting '(end 1 16 2 2) 2 4)
#(score-override-auto-beam-setting '(end 1 16 2 2) 3 4)
")
(d-SetSaved #f)

