;;;ToggleFlageolet
;use _instead of ^ to place harmonic below note
(ToggleChordDirective "Flageolet" LG-Flageolet "^\\flageolet" DENEMO_OVERRIDE_ABOVE)
(d-DirectivePut-chord-gy "Flageolet" 12)
;(d-DirectivePut-chord-gx "Flageolet" -2)
