  ;;; ShowOssia
(d-Directive-standalone "StartStaff")
(d-DirectivePut-standalone-postfix "StartStaff" "\\startStaff ")
(d-DirectivePut-standalone-display "StartStaff" "Show")
(d-DirectivePut-standalone-ty "StartStaff" 40)
(d-RefreshDisplay)